@@ -15,7 +15,7 @@
 /*   the documentation and/or other materials provided with the */
 /*   distribution. */
 
-/* * Neither the name of the phonopy project nor the names of its */
+/* * Neither the name of the spglib project nor the names of its */
 /*   contributors may be used to endorse or promote products derived */
 /*   from this software without specific prior written permission. */
 
@@ -44,7 +44,21 @@
 /* 'a++' generalized to an arbitrary increment. */
 /* Performs 'a += b' and returns the old value of a. */
 #ifndef SPG_POST_INCREMENT
-#define SPG_POST_INCREMENT(a, b) (a += (b), a - (b))
+#define SPG_POST_INCREMENT(a, b) ((a) += (b), (a) - (b))
+#endif
+
+/* deal with inline */
+#if   defined(_MSC_VER)
+# define OVL_INLINE __forceinline
+#elif defined(__clang__)
+# define OVL_INLINE __inline__ __attribute__((__always_inline__))
+#elif defined(__GNUC__) && \
+    ( defined(__GNUC_STDC_INLINE__) || \
+     (defined(__STDC__) && (__STDC_VERSION__ >= 199901L)) || \
+     (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 2)) )
+# define OVL_INLINE __inline__ __attribute__((__always_inline__))
+#else
+# define OVL_INLINE /* empty */
 #endif
 
 /* Note: data_out and data_in MUST NOT ALIAS. */
@@ -87,7 +101,7 @@ static int argsort_by_lattice_point_distance(int * perm,
                                              const int * types,
                                              double * distance_temp,
                                              void *argsort_work,
-                                             int size);
+                                             const int size);
 
 static OverlapChecker* overlap_checker_alloc(int size);
 
@@ -96,8 +110,71 @@ static int check_total_overlap_for_sorted(SPGCONST double lattice[3][3],
                                           SPGCONST double (*pos_rotated)[3],
                                           const int types_original[],
                                           const int types_rotated[],
-                                          int num_pos,
-                                          double symprec);
+                                          const int num_pos,
+                                          const double symprec);
+
+/* Note that some compilers apparently don't like it
+ * when you have a separate prototype with a function
+ * marked inline, so no prototypes here.
+ *
+ * As an aside, yes, significant performance is lost
+ * for large structures if these functions aren't inlined. */
+
+static OVL_INLINE double cartesian_norm(SPGCONST double lat[3][3],
+                                           const double v[3])
+{
+  double temp[3];
+  temp[0] = lat[0][0] * v[0] + lat[0][1] * v[1] + lat[0][2] * v[2];
+  temp[1] = lat[1][0] * v[0] + lat[1][1] * v[1] + lat[1][2] * v[2];
+  temp[2] = lat[2][0] * v[0] + lat[2][1] * v[1] + lat[2][2] * v[2];
+
+  return sqrt(temp[0] * temp[0] +
+              temp[1] * temp[1] +
+              temp[2] * temp[2]);
+}
+
+static OVL_INLINE int Nint(const double a)
+{
+  if (a < 0.0)
+    return (int) (a - 0.5);
+  else
+    return (int) (a + 0.5);
+}
+
+static OVL_INLINE int has_overlap(const double a[3],
+                                  const double b[3],
+                                  SPGCONST double lattice[3][3],
+                                  const double symprec)
+{
+  double v_diff[3];
+  v_diff[0] = a[0] - b[0];
+  v_diff[2] = a[2] - b[2];
+  v_diff[1] = a[1] - b[1];
+
+  v_diff[0] -= Nint(v_diff[0]);
+  v_diff[1] -= Nint(v_diff[1]);
+  v_diff[2] -= Nint(v_diff[2]);
+
+  if (cartesian_norm(lattice, v_diff) <= symprec) {
+    return 1;
+  } else {
+    return 0;
+  }
+}
+
+static OVL_INLINE int has_overlap_with_same_type(const double a[3],
+                                                 const double b[3],
+                                                 const int type_a,
+                                                 const int type_b,
+                                                 SPGCONST double lattice[3][3],
+                                                 const double symprec)
+{
+  if (type_a == type_b) {
+    return has_overlap(a, b, lattice, symprec);
+  } else {
+    return 0;
+  }
+}
 
 /* ------------------------------------- */
 /*          arg-sorting                  */
@@ -328,8 +405,8 @@ static void permute(void *data_out,
   void *write;
 
   for (i = 0; i < n; i++) {
-    read = data_in + perm[i] * value_size;
-    write = data_out + i * value_size;
+    read = (void *)((char *)data_in + perm[i] * value_size);
+    write = (void *)((char *)data_out + i * value_size);
     memcpy(write, read, value_size);
   }
 }
@@ -357,10 +434,14 @@ static OverlapChecker* overlap_checker_alloc(int size)
   int offset_pos_temp_1, offset_pos_temp_2, offset_distance_temp;
   int offset_perm_temp, offset_pos_sorted, offset_types_sorted, offset_lattice;
   int offset, blob_size;
-
+  char * chr_blob;
   OverlapChecker * checker;
+
+  chr_blob = NULL;
   checker = NULL;
 
+  /* checker->blob is going to contain lots of things. */
+  /* Compute its total size and the number of bytes before each thing. */
   offset = 0;
   offset_pos_temp_1 = SPG_POST_INCREMENT(offset, size * sizeof(double[3]));
   offset_pos_temp_2 = SPG_POST_INCREMENT(offset, size * sizeof(double[3]));
@@ -392,13 +473,18 @@ static OverlapChecker* overlap_checker_alloc(int size)
   }
 
   checker->size = size;
-  checker->pos_temp_1 = (double (*)[3])(checker->blob + offset_pos_temp_1);
-  checker->pos_temp_2 = (double (*)[3])(checker->blob + offset_pos_temp_2);
-  checker->distance_temp = (double *)(checker->blob + offset_distance_temp);
-  checker->perm_temp = (int *)(checker->blob + offset_perm_temp);
-  checker->lattice = (double (*)[3])(checker->blob + offset_lattice);
-  checker->pos_sorted  = (double (*)[3])(checker->blob + offset_pos_sorted);
-  checker->types_sorted = (int *)(checker->blob + offset_types_sorted);
+
+  /* Create the pointers to the things contained in checker->blob. */
+  /* The C spec doesn't allow arithmetic directly on 'void *', */
+  /* so a 'char *' is used. */
+  chr_blob = (char *)checker->blob;
+  checker->pos_temp_1 = (double (*)[3])(chr_blob + offset_pos_temp_1);
+  checker->pos_temp_2 = (double (*)[3])(chr_blob + offset_pos_temp_2);
+  checker->distance_temp = (double *)(chr_blob + offset_distance_temp);
+  checker->perm_temp = (int *)(chr_blob + offset_perm_temp);
+  checker->lattice = (double (*)[3])(chr_blob + offset_lattice);
+  checker->pos_sorted  = (double (*)[3])(chr_blob + offset_pos_sorted);
+  checker->types_sorted = (int *)(chr_blob + offset_types_sorted);
 
   return checker;
 }
@@ -468,12 +554,12 @@ static int check_possible_overlap(OverlapChecker *checker,
     /*  upper bounds on 'i'. For now though, brute force is good enough) */
     is_found = 0;
     for (i = 0; i < checker->size; i++) {
-      if (cel_is_overlap_with_same_type(pos_rot,
-                                        checker->pos_sorted[i],
-                                        type_rot,
-                                        checker->types_sorted[i],
-                                        checker->lattice,
-                                        symprec)) {
+      if (has_overlap_with_same_type(pos_rot,
+                                     checker->pos_sorted[i],
+                                     type_rot,
+                                     checker->types_sorted[i],
+                                     checker->lattice,
+                                     symprec)) {
         is_found = 1;
         break;
       }
@@ -531,12 +617,12 @@ static int check_total_overlap_for_sorted(SPGCONST double lattice[3][3],
         continue;
       }
 
-      if (cel_is_overlap_with_same_type(pos_original[i_orig],
-                                        pos_rotated[i_rot],
-                                        types_original[i_orig],
-                                        types_rotated[i_rot],
-                                        lattice,
-                                        symprec)) {
+      if (has_overlap_with_same_type(pos_original[i_orig],
+                                     pos_rotated[i_rot],
+                                     types_original[i_orig],
+                                     types_rotated[i_rot],
+                                     lattice,
+                                     symprec)) {
         found[i_rot] = 1;
         break;
       }
@@ -546,6 +632,8 @@ static int check_total_overlap_for_sorted(SPGCONST double lattice[3][3],
       /* We never hit the 'break'. */
       /* Failure; a position in pos_original does not */
       /* overlap with any position in pos_rotated. */
+      free(found);
+      found = NULL;
       return 0;
     }
   }
