@@ -15,7 +15,7 @@
 /*   the documentation and/or other materials provided with the */
 /*   distribution. */
 
-/* * Neither the name of the phonopy project nor the names of its */
+/* * Neither the name of the spglib project nor the names of its */
 /*   contributors may be used to endorse or promote products derived */
 /*   from this software without specific prior written permission. */
 
@@ -78,8 +78,8 @@ extern "C" {
   each translation is:
   [ t_a, t_b, t_c ]
 
-  symprec: Tolerance of atomic positions (in fractional coordinate)
-  in finding symmetry operations
+  symprec: Distance tolerance in Cartesian coordinates to find crystal
+           symmetry.
 
   ------------------------------------------------------------------
 
@@ -123,6 +123,8 @@ extern "C" {
     int *wyckoffs;
     char (*site_symmetry_symbols)[7];
     int *equivalent_atoms;
+    int *crystallographic_orbits;
+    double primitive_lattice[3][3];
     int *mapping_to_primitive;
     int n_std_atoms;
     double std_lattice[3][3];
@@ -237,6 +239,41 @@ extern "C" {
                                              const double symprec,
                                              const double angle_tolerance);
 
+/* Return 0 if failed */
+/* ``rotation`` and ``tranlation`` are used as input and output. */
+/* ``num_operations`` is the number of the symmetry operations of */
+/* input. */
+  int spg_get_symmetry_with_site_tensors(int rotation[][3][3],
+                                         double translation[][3],
+                                         int equivalent_atoms[],
+                                         double primitive_lattice[3][3],
+                                         int *spin_flips,
+                                         const int num_operations,
+                                         SPGCONST double lattice[3][3],
+                                         SPGCONST double position[][3],
+                                         const int types[],
+                                         const double *tensors,
+                                         const int tensor_rank,
+                                         const int num_atom,
+                                         const int is_magnetic,
+                                         const double symprec);
+
+  int spgat_get_symmetry_with_site_tensors(int rotation[][3][3],
+                                           double translation[][3],
+                                           int equivalent_atoms[],
+                                           double primitive_lattice[3][3],
+                                           int *spin_flips,
+                                           const int num_operations,
+                                           SPGCONST double lattice[3][3],
+                                           SPGCONST double position[][3],
+                                           const int types[],
+                                           const double *tensors,
+                                           const int tensor_rank,
+                                           const int num_atom,
+                                           const int is_magnetic,
+                                           const double symprec,
+                                           const double angle_tolerance);
+
 /* Space group type (hall_number) is searched from symmetry operations. */
   int spg_get_hall_number_from_symmetry(SPGCONST int rotation[][3][3],
                                         SPGCONST double translation[][3],
@@ -451,33 +488,33 @@ extern "C" {
 /* Rotation operations in reciprocal space ``rot_reciprocal`` are applied */
 /* to a grid address ``address_orig`` and resulting grid points are stored in */
 /* ``rot_grid_points``. Return 0 if failed. */
-  int spg_get_grid_points_by_rotations(int rot_grid_points[],
-                                       const int address_orig[3],
-                                       const int num_rot,
-                                       SPGCONST int rot_reciprocal[][3][3],
-                                       const int mesh[3],
-                                       const int is_shift[3]);
-  int spg_get_dense_grid_points_by_rotations(size_t rot_grid_points[],
-                                             const int address_orig[3],
-                                             const int num_rot,
-                                             SPGCONST int rot_reciprocal[][3][3],
-                                             const int mesh[3],
-                                             const int is_shift[3]);
+  void spg_get_grid_points_by_rotations(int rot_grid_points[],
+                                        const int address_orig[3],
+                                        const int num_rot,
+                                        SPGCONST int rot_reciprocal[][3][3],
+                                        const int mesh[3],
+                                        const int is_shift[3]);
+  void spg_get_dense_grid_points_by_rotations(size_t rot_grid_points[],
+                                              const int address_orig[3],
+                                              const int num_rot,
+                                              SPGCONST int rot_reciprocal[][3][3],
+                                              const int mesh[3],
+                                              const int is_shift[3]);
 
-  int spg_get_BZ_grid_points_by_rotations(int rot_grid_points[],
-                                          const int address_orig[3],
-                                          const int num_rot,
-                                          SPGCONST int rot_reciprocal[][3][3],
-                                          const int mesh[3],
-                                          const int is_shift[3],
-                                          const int bz_map[]);
-  int spg_get_dense_BZ_grid_points_by_rotations(size_t rot_grid_points[],
-                                                const int address_orig[3],
-                                                const int num_rot,
-                                                SPGCONST int rot_reciprocal[][3][3],
-                                                const int mesh[3],
-                                                const int is_shift[3],
-                                                const size_t bz_map[]);
+  void spg_get_BZ_grid_points_by_rotations(int rot_grid_points[],
+                                           const int address_orig[3],
+                                           const int num_rot,
+                                           SPGCONST int rot_reciprocal[][3][3],
+                                           const int mesh[3],
+                                           const int is_shift[3],
+                                           const int bz_map[]);
+  void spg_get_dense_BZ_grid_points_by_rotations(size_t rot_grid_points[],
+                                                 const int address_orig[3],
+                                                 const int num_rot,
+                                                 SPGCONST int rot_reciprocal[][3][3],
+                                                 const int mesh[3],
+                                                 const int is_shift[3],
+                                                 const size_t bz_map[]);
 
 /* Grid addresses are relocated inside Brillouin zone. */
 /* Number of ir-grid-points inside Brillouin zone is returned. */
@@ -514,14 +551,6 @@ extern "C" {
                                             SPGCONST double rec_lattice[3][3],
                                             const int is_shift[3]);
 
-  void spg_get_neighboring_grid_points(int relative_grid_points[],
-                                       const int grid_point,
-                                       SPGCONST int relative_grid_address[][3],
-                                       const int num_relative_grid_address,
-                                       const int mesh[3],
-                                       SPGCONST int bz_grid_address[][3],
-                                       const int bz_map[]);
-
 /*--------*/
 /* Niggli */
 /*--------*/
